package com.kim.apidoc.core.parser;

import com.github.javaparser.ast.body.ClassOrInterfaceDeclaration;
import com.github.javaparser.ast.body.EnumDeclaration;
import com.github.javaparser.ast.body.MethodDeclaration;
import com.github.javaparser.ast.comments.Comment;
import com.github.javaparser.ast.comments.JavadocComment;
import com.github.javaparser.ast.visitor.VoidVisitorAdapter;
import com.github.javaparser.javadoc.Javadoc;
import com.github.javaparser.javadoc.JavadocBlockTag;
import com.kim.apidoc.core.common.helper.OptionalHelper;
import com.kim.apidoc.core.schema.Chapter;
import com.kim.apidoc.core.schema.Node;
import com.kim.apidoc.core.schema.Project;
import com.kim.apidoc.core.schema.Section;

import java.util.Optional;

import static com.kim.apidoc.core.schema.Chapter.NODE_CHAPTER;

/**
 * The type Visitor parser.
 */
public class VisitorParser extends VoidVisitorAdapter<Node> {

    /**
     * The Parser strategy.
     */
    private ParserStrategy parserStrategy;

    /**
     * Sets parser strategy.
     *
     * @param parserStrategy the parser strategy
     */
    public void setParserStrategy(ParserStrategy parserStrategy) {
        this.parserStrategy = parserStrategy;
    }

    @Override
    public void visit(final EnumDeclaration enumDeclaration, final Node arg) {
        super.visit(enumDeclaration, arg);
    }

    /**
     * 类或者接口声明
     *
     * @param classOrInterfaceDeclaration
     * @param arg
     */
    @Override
    public void visit(final ClassOrInterfaceDeclaration classOrInterfaceDeclaration, final Node arg) {
        if (arg instanceof Project) {
            Project project = (Project) arg;
            // 章节
            Chapter chapter = new Chapter();
            classOrInterfaceDeclaration.getFullyQualifiedName().ifPresent(chapter::setId);
            chapter.setName(classOrInterfaceDeclaration.getNameAsString());
            classOrInterfaceDeclaration.getComment().ifPresent(chapter::accept);

            OptionalHelper.any(chapter.getTag("book"), chapter.getTag("group"))
                    .ifPresent(tag -> chapter.setBookName(tag.getContent()));

            if (parserStrategy.accept(classOrInterfaceDeclaration)) {
                chapter.setType(NODE_CHAPTER);
                parserStrategy.visit(classOrInterfaceDeclaration, chapter);
                project.addChapter(chapter);
                super.visit(classOrInterfaceDeclaration, chapter);
            } else if (isExistTag(classOrInterfaceDeclaration, "code")) {
                //chapter.setType(NODE_APPENDIX);
                //if (null==project.getAppendixChapter(chapter.getBookName())){
                //    project.addChapter(chapter);
                //}
                super.visit(classOrInterfaceDeclaration, arg);
            } else if (isExistTag(classOrInterfaceDeclaration, "resultData")) {
            }
        }
    }


    /**
     * 处理特殊的tag
     *
     * @param classOrInterfaceDeclaration the class or interface declaration
     * @param tag                         the tag
     * @return boolean boolean
     */
    private Boolean isExistTag(ClassOrInterfaceDeclaration classOrInterfaceDeclaration, String tag) {
        if (classOrInterfaceDeclaration.getComment().isPresent()) {
            Comment comment = classOrInterfaceDeclaration.getComment().get();
            if (!comment.isJavadocComment()) {
                return false;
            }
            Javadoc javadoc = comment.asJavadocComment().parse();

            Optional<JavadocBlockTag> blockTagOptional =
                    javadoc.getBlockTags().stream().filter(blockTag -> {
                        return tag.equalsIgnoreCase(blockTag.getTagName());
                    }).findFirst();
            if (blockTagOptional.isPresent()) {
                return true;
            }

        }
        return false;
    }

    @Override
    public void visit(JavadocComment n, Node arg) {
        super.visit(n, arg);
    }

    /**
     * 方法声明
     *
     * @param methodDeclaration
     * @param arg
     */
    @Override
    public void visit(final MethodDeclaration methodDeclaration, final Node arg) {
        if (arg instanceof Chapter && parserStrategy.accept(methodDeclaration)) {
            Chapter chapter = (Chapter) arg;
            Section section = new Section();
            section.setId(methodDeclaration.getNameAsString());
            section.setName(methodDeclaration.getNameAsString());
            section.setIndex(chapter.getSections().size());
            methodDeclaration.getComment().ifPresent(section::accept);

            parserStrategy.visit(methodDeclaration, chapter, section);
            chapter.getSections().add(section);
            super.visit(methodDeclaration, section);
        }
    }

}
