package com.kim.apidoc.core.schema;

import lombok.Getter;
import lombok.Setter;
import org.apache.commons.lang3.StringUtils;

import java.util.*;

import static com.kim.apidoc.core.schema.Chapter.NODE_APPENDIX;

/**
 * 项目
 *
 * @author fengyuchenglun
 * @version 1.0.0
 */
@Setter
@Getter
public class Project extends Node {

    /**
     * 版本
     */
    String version;

    /**
     * The Books.
     */
    Map<String, Book> books = new TreeMap<>();

    /**
     * Add chapter.
     *
     * @param chapter the chapter
     */
    public void addChapter(Chapter chapter) {
        if (Objects.isNull(chapter.getBookName())) {
            chapter.setBookName(Book.DEFAULT);
        }
        if (!books.containsKey(chapter.getBookName())) {
            books.put(chapter.getBookName(), new Book(chapter.getBookName()));
        }
        books.get(chapter.getBookName()).getChapters().add(chapter);
    }

    public Chapter getAppendixChapter(String bookName) {
        if (StringUtils.isBlank(bookName)) {
            bookName = Book.DEFAULT;
        }
        if (books.containsKey(bookName)) {
            Optional<Chapter> chapterOptional = books.get(bookName).getChapters().stream().filter(x -> NODE_APPENDIX.equals(x.getType())).findFirst();
            if (chapterOptional.isPresent()) {
                return chapterOptional.get();
            }
        }
        return null;
    }
}
